"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright 2019-2020 The Kubeflow Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var express_1 = __importDefault(require("express"));
var node_fetch_1 = __importDefault(require("node-fetch"));
var supertest_1 = __importDefault(require("supertest"));
var app_1 = require("./app");
var configs_1 = require("./configs");
var k8s_helper_1 = require("./k8s-helper");
var test_helper_1 = require("./integration-tests/test-helper");
jest.mock('node-fetch');
// TODO: move sections of tests here to individual files in `frontend/server/integration-tests/`
// for better organization and shorter/more focused tests.
var mockedFetch = node_fetch_1.default;
describe('UIServer apis', function () {
    var app;
    var tagName = '1.0.0';
    var commitHash = 'abcdefg';
    var _a = test_helper_1.commonSetup({ tagName: tagName, commitHash: commitHash }), argv = _a.argv, buildDate = _a.buildDate, indexHtmlContent = _a.indexHtmlContent;
    afterEach(function () {
        if (app) {
            app.close();
        }
    });
    describe('/', function () {
        it('responds with unmodified index.html if it is not a kubeflow deployment', function (done) {
            var expectedIndexHtml = "\n<html>\n<head>\n  <script>\n  window.KFP_FLAGS.DEPLOYMENT=null\n  window.KFP_FLAGS.HIDE_SIDENAV=false\n  </script>\n  <script id=\"kubeflow-client-placeholder\"></script>\n</head>\n</html>";
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/')
                .expect('Content-Type', 'text/html; charset=utf-8')
                .expect(200, expectedIndexHtml, done);
        });
        it('responds with a modified index.html if it is a kubeflow deployment and sets HIDE_SIDENAV', function (done) {
            var expectedIndexHtml = "\n<html>\n<head>\n  <script>\n  window.KFP_FLAGS.DEPLOYMENT=\"KUBEFLOW\"\n  window.KFP_FLAGS.HIDE_SIDENAV=true\n  </script>\n  <script id=\"kubeflow-client-placeholder\" src=\"/dashboard_lib.bundle.js\"></script>\n</head>\n</html>";
            var configs = configs_1.loadConfigs(argv, { DEPLOYMENT: 'kubeflow' });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/')
                .expect('Content-Type', 'text/html; charset=utf-8')
                .expect(200, expectedIndexHtml, done);
        });
        it('responds with flag DEPLOYMENT=MARKETPLACE if it is a marketplace deployment', function (done) {
            var expectedIndexHtml = "\n<html>\n<head>\n  <script>\n  window.KFP_FLAGS.DEPLOYMENT=\"MARKETPLACE\"\n  window.KFP_FLAGS.HIDE_SIDENAV=false\n  </script>\n  <script id=\"kubeflow-client-placeholder\"></script>\n</head>\n</html>";
            var configs = configs_1.loadConfigs(argv, { DEPLOYMENT: 'marketplace' });
            app = new app_1.UIServer(configs);
            var request = supertest_1.default(app.start());
            request
                .get('/')
                .expect('Content-Type', 'text/html; charset=utf-8')
                .expect(200, expectedIndexHtml, done);
        });
    });
    it('responds with flag HIDE_SIDENAV=false even when DEPLOYMENT=KUBEFLOW', function (done) {
        var expectedIndexHtml = "\n<html>\n<head>\n  <script>\n  window.KFP_FLAGS.DEPLOYMENT=\"KUBEFLOW\"\n  window.KFP_FLAGS.HIDE_SIDENAV=false\n  </script>\n  <script id=\"kubeflow-client-placeholder\" src=\"/dashboard_lib.bundle.js\"></script>\n</head>\n</html>";
        var configs = configs_1.loadConfigs(argv, { DEPLOYMENT: 'KUBEFLOW', HIDE_SIDENAV: 'false' });
        app = new app_1.UIServer(configs);
        var request = supertest_1.default(app.start());
        request
            .get('/')
            .expect('Content-Type', 'text/html; charset=utf-8')
            .expect(200, expectedIndexHtml, done);
    });
    describe('/apis/v1beta1/healthz', function () {
        it('responds with apiServerReady to be false if ml-pipeline api server is not ready.', function (done) {
            node_fetch_1.default.mockImplementationOnce(function (_url, _opt) { return ({
                json: function () { return Promise.reject('Unknown error'); },
            }); });
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            supertest_1.default(app.start())
                .get('/apis/v1beta1/healthz')
                .expect(200, {
                apiServerReady: false,
                buildDate: buildDate,
                frontendCommitHash: commitHash,
                frontendTagName: tagName,
            }, done);
        });
        it('responds with both ui server and ml-pipeline api state if ml-pipeline api server is also ready.', function (done) {
            node_fetch_1.default.mockImplementationOnce(function (_url, _opt) { return ({
                json: function () {
                    return Promise.resolve({
                        commit_sha: 'commit_sha',
                        tag_name: '1.0.0',
                        multi_user: false,
                    });
                },
            }); });
            var configs = configs_1.loadConfigs(argv, {});
            app = new app_1.UIServer(configs);
            supertest_1.default(app.start())
                .get('/apis/v1beta1/healthz')
                .expect(200, {
                apiServerCommitHash: 'commit_sha',
                apiServerTagName: '1.0.0',
                apiServerMultiUser: false,
                multi_user: false,
                apiServerReady: true,
                buildDate: buildDate,
                frontendCommitHash: commitHash,
                frontendTagName: tagName,
            }, done);
        });
    });
    describe('/system', function () {
        describe('/cluster-name', function () {
            it('responds with cluster name data from gke metadata', function (done) {
                mockedFetch.mockImplementationOnce(function (url, _opts) {
                    return url === 'http://metadata/computeMetadata/v1/instance/attributes/cluster-name'
                        ? Promise.resolve({ ok: true, text: function () { return Promise.resolve('test-cluster'); } })
                        : Promise.reject('Unexpected request');
                });
                app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
                var request = supertest_1.default(app.start());
                request
                    .get('/system/cluster-name')
                    .expect('Content-Type', 'text/html; charset=utf-8')
                    .expect(200, 'test-cluster', done);
            });
            it('responds with 500 status code if corresponding endpoint is not ok', function (done) {
                mockedFetch.mockImplementationOnce(function (url, _opts) {
                    return url === 'http://metadata/computeMetadata/v1/instance/attributes/cluster-name'
                        ? Promise.resolve({ ok: false, text: function () { return Promise.resolve('404 not found'); } })
                        : Promise.reject('Unexpected request');
                });
                app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
                var request = supertest_1.default(app.start());
                request.get('/system/cluster-name').expect(500, 'Failed fetching GKE cluster name', done);
            });
            it('responds with endpoint disabled if DISABLE_GKE_METADATA env is true', function (done) {
                var configs = configs_1.loadConfigs(argv, { DISABLE_GKE_METADATA: 'true' });
                app = new app_1.UIServer(configs);
                var request = supertest_1.default(app.start());
                request
                    .get('/system/cluster-name')
                    .expect(500, 'GKE metadata endpoints are disabled.', done);
            });
        });
        describe('/project-id', function () {
            it('responds with project id data from gke metadata', function (done) {
                mockedFetch.mockImplementationOnce(function (url, _opts) {
                    return url === 'http://metadata/computeMetadata/v1/project/project-id'
                        ? Promise.resolve({ ok: true, text: function () { return Promise.resolve('test-project'); } })
                        : Promise.reject('Unexpected request');
                });
                app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
                var request = supertest_1.default(app.start());
                request.get('/system/project-id').expect(200, 'test-project', done);
            });
            it('responds with 500 status code if metadata request is not ok', function (done) {
                mockedFetch.mockImplementationOnce(function (url, _opts) {
                    return url === 'http://metadata/computeMetadata/v1/project/project-id'
                        ? Promise.resolve({ ok: false, text: function () { return Promise.resolve('404 not found'); } })
                        : Promise.reject('Unexpected request');
                });
                app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
                var request = supertest_1.default(app.start());
                request.get('/system/project-id').expect(500, 'Failed fetching GKE project id', done);
            });
            it('responds with endpoint disabled if DISABLE_GKE_METADATA env is true', function (done) {
                app = new app_1.UIServer(configs_1.loadConfigs(argv, { DISABLE_GKE_METADATA: 'true' }));
                var request = supertest_1.default(app.start());
                request.get('/system/project-id').expect(500, 'GKE metadata endpoints are disabled.', done);
            });
        });
    });
    describe('/k8s/pod', function () {
        var request;
        beforeEach(function () {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            request = supertest_1.default(app.start());
        });
        it('asks for podname if not provided', function (done) {
            request.get('/k8s/pod').expect(422, 'podname argument is required', done);
        });
        it('asks for podnamespace if not provided', function (done) {
            request
                .get('/k8s/pod?podname=test-pod')
                .expect(422, 'podnamespace argument is required', done);
        });
        it('responds with pod info in JSON', function (done) {
            var readPodSpy = jest.spyOn(k8s_helper_1.TEST_ONLY.k8sV1Client, 'readNamespacedPod');
            readPodSpy.mockImplementation(function () {
                return Promise.resolve({
                    body: { kind: 'Pod' },
                });
            });
            request
                .get('/k8s/pod?podname=test-pod&podnamespace=test-ns')
                .expect(200, '{"kind":"Pod"}', function (err) {
                expect(readPodSpy).toHaveBeenCalledWith('test-pod', 'test-ns');
                done(err);
            });
        });
        it('responds with error when failed to retrieve pod info', function (done) {
            var readPodSpy = jest.spyOn(k8s_helper_1.TEST_ONLY.k8sV1Client, 'readNamespacedPod');
            readPodSpy.mockImplementation(function () {
                return Promise.reject({
                    body: {
                        message: 'pod not found',
                        code: 404,
                    },
                });
            });
            var spyError = jest.spyOn(console, 'error').mockImplementation(function () { return null; });
            request
                .get('/k8s/pod?podname=test-pod&podnamespace=test-ns')
                .expect(500, 'Could not get pod test-pod in namespace test-ns: pod not found', function () {
                expect(spyError).toHaveBeenCalledTimes(1);
                done();
            });
        });
    });
    describe('/k8s/pod/events', function () {
        var request;
        beforeEach(function () {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            request = supertest_1.default(app.start());
        });
        it('asks for podname if not provided', function (done) {
            request.get('/k8s/pod/events').expect(422, 'podname argument is required', done);
        });
        it('asks for podnamespace if not provided', function (done) {
            request
                .get('/k8s/pod/events?podname=test-pod')
                .expect(422, 'podnamespace argument is required', done);
        });
        it('responds with pod info in JSON', function (done) {
            var listEventSpy = jest.spyOn(k8s_helper_1.TEST_ONLY.k8sV1Client, 'listNamespacedEvent');
            listEventSpy.mockImplementation(function () {
                return Promise.resolve({
                    body: { kind: 'EventList' },
                });
            });
            request
                .get('/k8s/pod/events?podname=test-pod&podnamespace=test-ns')
                .expect(200, '{"kind":"EventList"}', function (err) {
                expect(listEventSpy).toHaveBeenCalledWith('test-ns', undefined, undefined, undefined, 'involvedObject.namespace=test-ns,involvedObject.name=test-pod,involvedObject.kind=Pod');
                done(err);
            });
        });
        it('responds with error when failed to retrieve pod info', function (done) {
            var listEventSpy = jest.spyOn(k8s_helper_1.TEST_ONLY.k8sV1Client, 'listNamespacedEvent');
            listEventSpy.mockImplementation(function () {
                return Promise.reject({
                    body: {
                        message: 'no events',
                        code: 404,
                    },
                });
            });
            var spyError = jest.spyOn(console, 'error').mockImplementation(function () { return null; });
            request
                .get('/k8s/pod/events?podname=test-pod&podnamespace=test-ns')
                .expect(500, 'Error when listing pod events for pod "test-pod" in "test-ns" namespace: no events', function (err) {
                expect(spyError).toHaveBeenCalledTimes(1);
                done(err);
            });
        });
    });
    // TODO: Add integration tests for k8s helper related endpoints
    // describe('/k8s/pod/logs', () => {});
    describe('/apis/v1beta1/', function () {
        var request;
        var kfpApiServer;
        beforeEach(function () {
            var kfpApiPort = 3001;
            kfpApiServer = express_1.default()
                .all('/*', function (_, res) {
                res.status(200).send('KFP API is working');
            })
                .listen(kfpApiPort);
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {
                ML_PIPELINE_SERVICE_PORT: "" + kfpApiPort,
                ML_PIPELINE_SERVICE_HOST: 'localhost',
            }));
            request = supertest_1.default(app.start());
        });
        afterEach(function () {
            if (kfpApiServer) {
                kfpApiServer.close();
            }
        });
        it('rejects reportMetrics because it is not public kfp api', function (done) {
            var runId = 'a-random-run-id';
            request
                .post("/apis/v1beta1/runs/" + runId + ":reportMetrics")
                .expect(403, '/apis/v1beta1/runs/a-random-run-id:reportMetrics endpoint is not meant for external usage.', done);
        });
        it('rejects reportWorkflow because it is not public kfp api', function (done) {
            var workflowId = 'a-random-workflow-id';
            request
                .post("/apis/v1beta1/workflows/" + workflowId)
                .expect(403, '/apis/v1beta1/workflows/a-random-workflow-id endpoint is not meant for external usage.', done);
        });
        it('rejects reportScheduledWorkflow because it is not public kfp api', function (done) {
            var swf = 'a-random-swf-id';
            request
                .post("/apis/v1beta1/scheduledworkflows/" + swf)
                .expect(403, '/apis/v1beta1/scheduledworkflows/a-random-swf-id endpoint is not meant for external usage.', done);
        });
        it('does not reject similar apis', function (done) {
            request // use reportMetrics as runId to see if it can confuse route parsing
                .post("/apis/v1beta1/runs/xxx-reportMetrics:archive")
                .expect(200, 'KFP API is working', done);
        });
        it('proxies other run apis', function (done) {
            request
                .post("/apis/v1beta1/runs/a-random-run-id:archive")
                .expect(200, 'KFP API is working', done);
        });
    });
});
//# sourceMappingURL=app.test.js.map