"use strict";
// Copyright 2021 The Kubeflow Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var express_1 = __importDefault(require("express"));
var fs = __importStar(require("fs"));
var path = __importStar(require("path"));
var supertest_1 = __importDefault(require("supertest"));
var app_1 = require("../app");
var configs_1 = require("../configs");
var k8s_helper_1 = require("../k8s-helper");
var test_helper_1 = require("./test-helper");
beforeEach(function () {
    jest.spyOn(global.console, 'info').mockImplementation();
    jest.spyOn(global.console, 'log').mockImplementation();
    jest.spyOn(global.console, 'debug').mockImplementation();
});
describe('/apps/tensorboard', function () {
    var app;
    afterEach(function () {
        if (app) {
            app.close();
        }
    });
    var tagName = '1.0.0';
    var commitHash = 'abcdefg';
    var argv = test_helper_1.commonSetup({ tagName: tagName, commitHash: commitHash }).argv;
    var POD_TEMPLATE_SPEC = {
        spec: {
            containers: [
                {
                    volumeMounts: [
                        {
                            name: 'tensorboard',
                            mountPath: '/logs',
                        },
                        {
                            name: 'data',
                            subPath: 'tensorboard',
                            mountPath: '/data',
                        },
                    ],
                },
            ],
            volumes: [
                {
                    name: 'tensorboard',
                    persistentVolumeClaim: {
                        claimName: 'logs',
                    },
                },
                {
                    name: 'data',
                    persistentVolumeClaim: {
                        claimName: 'data',
                    },
                },
            ],
        },
    };
    var k8sGetCustomObjectSpy;
    var k8sDeleteCustomObjectSpy;
    var k8sCreateCustomObjectSpy;
    var kfpApiServer;
    function newGetTensorboardResponse(_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.name, name = _c === void 0 ? 'viewer-example' : _c, _d = _b.logDir, logDir = _d === void 0 ? 'log-dir-example' : _d, _e = _b.tensorflowImage, tensorflowImage = _e === void 0 ? 'tensorflow:2.0.0' : _e, _f = _b.type, type = _f === void 0 ? 'tensorboard' : _f;
        return {
            response: undefined,
            body: {
                metadata: {
                    name: name,
                },
                spec: {
                    tensorboardSpec: { logDir: logDir, tensorflowImage: tensorflowImage },
                    type: type,
                },
            },
        };
    }
    beforeEach(function () {
        k8sGetCustomObjectSpy = jest.spyOn(k8s_helper_1.TEST_ONLY.k8sV1CustomObjectClient, 'getNamespacedCustomObject');
        k8sDeleteCustomObjectSpy = jest.spyOn(k8s_helper_1.TEST_ONLY.k8sV1CustomObjectClient, 'deleteNamespacedCustomObject');
        k8sCreateCustomObjectSpy = jest.spyOn(k8s_helper_1.TEST_ONLY.k8sV1CustomObjectClient, 'createNamespacedCustomObject');
    });
    afterEach(function () {
        if (kfpApiServer) {
            kfpApiServer.close();
        }
    });
    describe('get', function () {
        it('requires logdir for get tensorboard', function (done) {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .get('/apps/tensorboard')
                .expect(400, 'logdir argument is required', done);
        });
        it('requires namespace for get tensorboard', function (done) {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .get('/apps/tensorboard?logdir=some-log-dir')
                .expect(400, 'namespace argument is required', done);
        });
        it('does not crash with a weird query', function (done) {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            k8sGetCustomObjectSpy.mockImplementation(function () { return Promise.resolve(newGetTensorboardResponse()); });
            // The special case is that, decodeURIComponent('%2') throws an
            // exception, so this can verify handler doesn't do extra
            // decodeURIComponent on queries.
            var weirdLogDir = encodeURIComponent('%2');
            supertest_1.default(app.start())
                .get("/apps/tensorboard?logdir=" + weirdLogDir + "&namespace=test-ns")
                .expect(200, done);
        });
        function setupMockKfpApiService(_a) {
            var _b = (_a === void 0 ? {} : _a).port, port = _b === void 0 ? 3001 : _b;
            var receivedHeaders = [];
            kfpApiServer = express_1.default()
                .get('/apis/v1beta1/auth', function (req, res) {
                receivedHeaders.push(req.headers);
                res.status(200).send('{}'); // Authorized
            })
                .listen(port);
            return { receivedHeaders: receivedHeaders, host: 'localhost', port: port };
        }
        it('authorizes user requests from KFP auth api', function (done) {
            var _a = setupMockKfpApiService(), receivedHeaders = _a.receivedHeaders, host = _a.host, port = _a.port;
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {
                ENABLE_AUTHZ: 'true',
                ML_PIPELINE_SERVICE_PORT: "" + port,
                ML_PIPELINE_SERVICE_HOST: host,
            }));
            k8sGetCustomObjectSpy.mockImplementation(function () { return Promise.resolve(newGetTensorboardResponse()); });
            supertest_1.default(app.start())
                .get("/apps/tensorboard?logdir=some-log-dir&namespace=test-ns")
                .set('x-goog-authenticated-user-email', 'accounts.google.com:user@google.com')
                .expect(200, function (err) {
                expect(receivedHeaders).toHaveLength(1);
                expect(receivedHeaders[0]).toMatchInlineSnapshot("\n              Object {\n                \"accept\": \"*/*\",\n                \"accept-encoding\": \"gzip,deflate\",\n                \"connection\": \"close\",\n                \"host\": \"localhost:3001\",\n                \"user-agent\": \"node-fetch/1.0 (+https://github.com/bitinn/node-fetch)\",\n                \"x-goog-authenticated-user-email\": \"accounts.google.com:user@google.com\",\n              }\n            ");
                done(err);
            });
        });
        it('uses configured KUBEFLOW_USERID_HEADER for user identity', function (done) {
            var _a = setupMockKfpApiService(), receivedHeaders = _a.receivedHeaders, host = _a.host, port = _a.port;
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {
                ENABLE_AUTHZ: 'true',
                KUBEFLOW_USERID_HEADER: 'x-kubeflow-userid',
                ML_PIPELINE_SERVICE_PORT: "" + port,
                ML_PIPELINE_SERVICE_HOST: host,
            }));
            k8sGetCustomObjectSpy.mockImplementation(function () { return Promise.resolve(newGetTensorboardResponse()); });
            supertest_1.default(app.start())
                .get("/apps/tensorboard?logdir=some-log-dir&namespace=test-ns")
                .set('x-kubeflow-userid', 'user@kubeflow.org')
                .expect(200, function (err) {
                expect(receivedHeaders).toHaveLength(1);
                expect(receivedHeaders[0]).toHaveProperty('x-kubeflow-userid', 'user@kubeflow.org');
                done(err);
            });
        });
        it('rejects user requests when KFP auth api rejected', function (done) {
            var errorSpy = jest.spyOn(console, 'error');
            errorSpy.mockImplementation();
            var apiServerPort = 3001;
            kfpApiServer = express_1.default()
                .get('/apis/v1beta1/auth', function (_, res) {
                res.status(400).send(JSON.stringify({
                    error: 'User xxx is not unauthorized to list viewers',
                    details: ['unauthorized', 'callstack'],
                }));
            })
                .listen(apiServerPort);
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {
                ENABLE_AUTHZ: 'true',
                ML_PIPELINE_SERVICE_PORT: "" + apiServerPort,
                ML_PIPELINE_SERVICE_HOST: 'localhost',
            }));
            k8sGetCustomObjectSpy.mockImplementation(function () { return Promise.resolve(newGetTensorboardResponse()); });
            supertest_1.default(app.start())
                .get("/apps/tensorboard?logdir=some-log-dir&namespace=test-ns")
                .expect(401, 'User is not authorized to GET VIEWERS in namespace test-ns: User xxx is not unauthorized to list viewers', function (err) {
                expect(errorSpy).toHaveBeenCalledTimes(1);
                expect(errorSpy).toHaveBeenCalledWith('User is not authorized to GET VIEWERS in namespace test-ns: User xxx is not unauthorized to list viewers', ['unauthorized', 'callstack']);
                done(err);
            });
        });
        it('gets tensorboard url, version and image', function (done) {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            k8sGetCustomObjectSpy.mockImplementation(function () {
                return Promise.resolve(newGetTensorboardResponse({
                    name: 'viewer-abcdefg',
                    logDir: 'log-dir-1',
                    tensorflowImage: 'tensorflow:2.0.0',
                }));
            });
            supertest_1.default(app.start())
                .get("/apps/tensorboard?logdir=" + encodeURIComponent('log-dir-1') + "&namespace=test-ns")
                .expect(200, JSON.stringify({
                podAddress: 'http://viewer-abcdefg-service.test-ns.svc.cluster.local:80/tensorboard/viewer-abcdefg/',
                tfVersion: '2.0.0',
                image: 'tensorflow:2.0.0',
            }), function (err) {
                expect(k8sGetCustomObjectSpy.mock.calls[0]).toMatchInlineSnapshot("\n                              Array [\n                                \"kubeflow.org\",\n                                \"v1beta1\",\n                                \"test-ns\",\n                                \"viewers\",\n                                \"viewer-5e1404e679e27b0f0b8ecee8fe515830eaa736c5\",\n                              ]\n                          ");
                done(err);
            });
        });
    });
    describe('post (create)', function () {
        it('requires logdir', function (done) {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .post('/apps/tensorboard')
                .expect(400, 'logdir argument is required', done);
        });
        it('requires namespace', function (done) {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .post('/apps/tensorboard?logdir=some-log-dir')
                .expect(400, 'namespace argument is required', done);
        });
        it('requires tfversion or image', function (done) {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .post('/apps/tensorboard?logdir=some-log-dir&namespace=test-ns')
                .expect(400, 'missing required argument: tfversion (tensorflow version) or image', done);
        });
        it('creates tensorboard viewer custom object and waits for it', function (done) {
            var getRequestCount = 0;
            k8sGetCustomObjectSpy.mockImplementation(function () {
                ++getRequestCount;
                switch (getRequestCount) {
                    case 1:
                        return Promise.reject('Not found');
                    case 2:
                        return Promise.resolve(newGetTensorboardResponse({
                            name: 'viewer-abcdefg',
                            logDir: 'log-dir-1',
                            tensorflowImage: 'tensorflow:2.0.0',
                        }));
                    default:
                        throw new Error('only expected to be called twice in this test');
                }
            });
            k8sCreateCustomObjectSpy.mockImplementation(function () { return Promise.resolve(); });
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .post("/apps/tensorboard?logdir=" + encodeURIComponent('log-dir-1') + "&namespace=test-ns&tfversion=2.0.0")
                .expect(200, 'http://viewer-abcdefg-service.test-ns.svc.cluster.local:80/tensorboard/viewer-abcdefg/', function (err) {
                expect(k8sGetCustomObjectSpy.mock.calls[0]).toMatchInlineSnapshot("\n                Array [\n                  \"kubeflow.org\",\n                  \"v1beta1\",\n                  \"test-ns\",\n                  \"viewers\",\n                  \"viewer-5e1404e679e27b0f0b8ecee8fe515830eaa736c5\",\n                ]\n              ");
                expect(k8sCreateCustomObjectSpy.mock.calls[0]).toMatchInlineSnapshot("\n                Array [\n                  \"kubeflow.org\",\n                  \"v1beta1\",\n                  \"test-ns\",\n                  \"viewers\",\n                  Object {\n                    \"apiVersion\": \"kubeflow.org/v1beta1\",\n                    \"kind\": \"Viewer\",\n                    \"metadata\": Object {\n                      \"name\": \"viewer-5e1404e679e27b0f0b8ecee8fe515830eaa736c5\",\n                      \"namespace\": \"test-ns\",\n                    },\n                    \"spec\": Object {\n                      \"podTemplateSpec\": Object {\n                        \"spec\": Object {\n                          \"containers\": Array [\n                            Object {},\n                          ],\n                        },\n                      },\n                      \"tensorboardSpec\": Object {\n                        \"logDir\": \"log-dir-1\",\n                        \"tensorflowImage\": \"tensorflow/tensorflow:2.0.0\",\n                      },\n                      \"type\": \"tensorboard\",\n                    },\n                  },\n                ]\n              ");
                expect(k8sGetCustomObjectSpy.mock.calls[1]).toMatchInlineSnapshot("\n                Array [\n                  \"kubeflow.org\",\n                  \"v1beta1\",\n                  \"test-ns\",\n                  \"viewers\",\n                  \"viewer-5e1404e679e27b0f0b8ecee8fe515830eaa736c5\",\n                ]\n              ");
                done(err);
            });
        });
        it('creates tensorboard viewer with specified image', function (done) {
            var getRequestCount = 0;
            var image = 'gcr.io/deeplearning-platform-release/tf2-cpu.2-4';
            k8sGetCustomObjectSpy.mockImplementation(function () {
                ++getRequestCount;
                switch (getRequestCount) {
                    case 1:
                        return Promise.reject('Not found');
                    case 2:
                        return Promise.resolve(newGetTensorboardResponse({
                            tensorflowImage: image,
                        }));
                    default:
                        throw new Error('only expected to be called twice in this test');
                }
            });
            k8sCreateCustomObjectSpy.mockImplementation(function () { return Promise.resolve(); });
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .post("/apps/tensorboard" + test_helper_1.buildQuery({ logdir: 'log-dir-1', namespace: 'test-ns', image: image }))
                .expect(200, function (err) {
                expect(k8sCreateCustomObjectSpy.mock.calls[0][4].spec.tensorboardSpec.tensorflowImage).toEqual(image);
                done(err);
            });
        });
        it('creates tensorboard viewer with exist volume', function (done) {
            var getRequestCount = 0;
            k8sGetCustomObjectSpy.mockImplementation(function () {
                ++getRequestCount;
                switch (getRequestCount) {
                    case 1:
                        return Promise.reject('Not found');
                    case 2:
                        return Promise.resolve(newGetTensorboardResponse({
                            name: 'viewer-abcdefg',
                            logDir: 'Series1:/logs/log-dir-1,Series2:/logs/log-dir-2',
                            tensorflowImage: 'tensorflow:2.0.0',
                        }));
                    default:
                        throw new Error('only expected to be called twice in this test');
                }
            });
            k8sCreateCustomObjectSpy.mockImplementation(function () { return Promise.resolve(); });
            var tempPath = path.join(test_helper_1.mkTempDir(), 'config.json');
            fs.writeFileSync(tempPath, JSON.stringify(POD_TEMPLATE_SPEC));
            app = new app_1.UIServer(configs_1.loadConfigs(argv, { VIEWER_TENSORBOARD_POD_TEMPLATE_SPEC_PATH: tempPath }));
            supertest_1.default(app.start())
                .post("/apps/tensorboard?logdir=" + encodeURIComponent('Series1:volume://tensorboard/log-dir-1,Series2:volume://tensorboard/log-dir-2') + "&namespace=test-ns&tfversion=2.0.0")
                .expect(200, 'http://viewer-abcdefg-service.test-ns.svc.cluster.local:80/tensorboard/viewer-abcdefg/', function (err) {
                expect(k8sGetCustomObjectSpy.mock.calls[0]).toMatchInlineSnapshot("\n                Array [\n                  \"kubeflow.org\",\n                  \"v1beta1\",\n                  \"test-ns\",\n                  \"viewers\",\n                  \"viewer-a800f945f0934d978f9cce9959b82ff44dac8493\",\n                ]\n              ");
                expect(k8sCreateCustomObjectSpy.mock.calls[0]).toMatchInlineSnapshot("\n                Array [\n                  \"kubeflow.org\",\n                  \"v1beta1\",\n                  \"test-ns\",\n                  \"viewers\",\n                  Object {\n                    \"apiVersion\": \"kubeflow.org/v1beta1\",\n                    \"kind\": \"Viewer\",\n                    \"metadata\": Object {\n                      \"name\": \"viewer-a800f945f0934d978f9cce9959b82ff44dac8493\",\n                      \"namespace\": \"test-ns\",\n                    },\n                    \"spec\": Object {\n                      \"podTemplateSpec\": Object {\n                        \"spec\": Object {\n                          \"containers\": Array [\n                            Object {\n                              \"volumeMounts\": Array [\n                                Object {\n                                  \"mountPath\": \"/logs\",\n                                  \"name\": \"tensorboard\",\n                                },\n                                Object {\n                                  \"mountPath\": \"/data\",\n                                  \"name\": \"data\",\n                                  \"subPath\": \"tensorboard\",\n                                },\n                              ],\n                            },\n                          ],\n                          \"volumes\": Array [\n                            Object {\n                              \"name\": \"tensorboard\",\n                              \"persistentVolumeClaim\": Object {\n                                \"claimName\": \"logs\",\n                              },\n                            },\n                            Object {\n                              \"name\": \"data\",\n                              \"persistentVolumeClaim\": Object {\n                                \"claimName\": \"data\",\n                              },\n                            },\n                          ],\n                        },\n                      },\n                      \"tensorboardSpec\": Object {\n                        \"logDir\": \"Series1:/logs/log-dir-1,Series2:/logs/log-dir-2\",\n                        \"tensorflowImage\": \"tensorflow/tensorflow:2.0.0\",\n                      },\n                      \"type\": \"tensorboard\",\n                    },\n                  },\n                ]\n              ");
                expect(k8sGetCustomObjectSpy.mock.calls[1]).toMatchInlineSnapshot("\n                Array [\n                  \"kubeflow.org\",\n                  \"v1beta1\",\n                  \"test-ns\",\n                  \"viewers\",\n                  \"viewer-a800f945f0934d978f9cce9959b82ff44dac8493\",\n                ]\n              ");
                done(err);
            });
        });
        it('creates tensorboard viewer with exist subPath volume', function (done) {
            var getRequestCount = 0;
            k8sGetCustomObjectSpy.mockImplementation(function () {
                ++getRequestCount;
                switch (getRequestCount) {
                    case 1:
                        return Promise.reject('Not found');
                    case 2:
                        return Promise.resolve(newGetTensorboardResponse({
                            name: 'viewer-abcdefg',
                            logDir: 'Series1:/data/log-dir-1,Series2:/data/log-dir-2',
                            tensorflowImage: 'tensorflow:2.0.0',
                        }));
                    default:
                        throw new Error('only expected to be called twice in this test');
                }
            });
            k8sCreateCustomObjectSpy.mockImplementation(function () { return Promise.resolve(); });
            var tempPath = path.join(test_helper_1.mkTempDir(), 'config.json');
            fs.writeFileSync(tempPath, JSON.stringify(POD_TEMPLATE_SPEC));
            app = new app_1.UIServer(configs_1.loadConfigs(argv, { VIEWER_TENSORBOARD_POD_TEMPLATE_SPEC_PATH: tempPath }));
            supertest_1.default(app.start())
                .post("/apps/tensorboard?logdir=" + encodeURIComponent('Series1:volume://data/tensorboard/log-dir-1,Series2:volume://data/tensorboard/log-dir-2') + "&namespace=test-ns&tfversion=2.0.0")
                .expect(200, 'http://viewer-abcdefg-service.test-ns.svc.cluster.local:80/tensorboard/viewer-abcdefg/', function (err) {
                expect(k8sGetCustomObjectSpy.mock.calls[0]).toMatchInlineSnapshot("\n                Array [\n                  \"kubeflow.org\",\n                  \"v1beta1\",\n                  \"test-ns\",\n                  \"viewers\",\n                  \"viewer-82d7d06a6ecb1e4dcba66d06b884d6445a88e4ca\",\n                ]\n              ");
                expect(k8sCreateCustomObjectSpy.mock.calls[0]).toMatchInlineSnapshot("\n                Array [\n                  \"kubeflow.org\",\n                  \"v1beta1\",\n                  \"test-ns\",\n                  \"viewers\",\n                  Object {\n                    \"apiVersion\": \"kubeflow.org/v1beta1\",\n                    \"kind\": \"Viewer\",\n                    \"metadata\": Object {\n                      \"name\": \"viewer-82d7d06a6ecb1e4dcba66d06b884d6445a88e4ca\",\n                      \"namespace\": \"test-ns\",\n                    },\n                    \"spec\": Object {\n                      \"podTemplateSpec\": Object {\n                        \"spec\": Object {\n                          \"containers\": Array [\n                            Object {\n                              \"volumeMounts\": Array [\n                                Object {\n                                  \"mountPath\": \"/logs\",\n                                  \"name\": \"tensorboard\",\n                                },\n                                Object {\n                                  \"mountPath\": \"/data\",\n                                  \"name\": \"data\",\n                                  \"subPath\": \"tensorboard\",\n                                },\n                              ],\n                            },\n                          ],\n                          \"volumes\": Array [\n                            Object {\n                              \"name\": \"tensorboard\",\n                              \"persistentVolumeClaim\": Object {\n                                \"claimName\": \"logs\",\n                              },\n                            },\n                            Object {\n                              \"name\": \"data\",\n                              \"persistentVolumeClaim\": Object {\n                                \"claimName\": \"data\",\n                              },\n                            },\n                          ],\n                        },\n                      },\n                      \"tensorboardSpec\": Object {\n                        \"logDir\": \"Series1:/data/log-dir-1,Series2:/data/log-dir-2\",\n                        \"tensorflowImage\": \"tensorflow/tensorflow:2.0.0\",\n                      },\n                      \"type\": \"tensorboard\",\n                    },\n                  },\n                ]\n              ");
                expect(k8sGetCustomObjectSpy.mock.calls[1]).toMatchInlineSnapshot("\n                Array [\n                  \"kubeflow.org\",\n                  \"v1beta1\",\n                  \"test-ns\",\n                  \"viewers\",\n                  \"viewer-82d7d06a6ecb1e4dcba66d06b884d6445a88e4ca\",\n                ]\n              ");
                done(err);
            });
        });
        it('creates tensorboard viewer with not exist volume and return error', function (done) {
            var errorSpy = jest.spyOn(console, 'error');
            errorSpy.mockImplementation();
            k8sGetCustomObjectSpy.mockImplementation(function () {
                return Promise.reject('Not found');
            });
            var tempPath = path.join(test_helper_1.mkTempDir(), 'config.json');
            fs.writeFileSync(tempPath, JSON.stringify(POD_TEMPLATE_SPEC));
            app = new app_1.UIServer(configs_1.loadConfigs(argv, { VIEWER_TENSORBOARD_POD_TEMPLATE_SPEC_PATH: tempPath }));
            supertest_1.default(app.start())
                .post("/apps/tensorboard?logdir=" + encodeURIComponent('volume://notexistvolume/logs/log-dir-1') + "&namespace=test-ns&tfversion=2.0.0")
                .expect(500, "Failed to start Tensorboard app: Cannot find file \"volume://notexistvolume/logs/log-dir-1\" in pod \"unknown\": volume \"notexistvolume\" not configured", function (err) {
                expect(errorSpy).toHaveBeenCalledTimes(1);
                done(err);
            });
        });
        it('creates tensorboard viewer with not exist subPath volume mount and return error', function (done) {
            var errorSpy = jest.spyOn(console, 'error');
            errorSpy.mockImplementation();
            k8sGetCustomObjectSpy.mockImplementation(function () {
                return Promise.reject('Not found');
            });
            var tempPath = path.join(test_helper_1.mkTempDir(), 'config.json');
            fs.writeFileSync(tempPath, JSON.stringify(POD_TEMPLATE_SPEC));
            app = new app_1.UIServer(configs_1.loadConfigs(argv, { VIEWER_TENSORBOARD_POD_TEMPLATE_SPEC_PATH: tempPath }));
            supertest_1.default(app.start())
                .post("/apps/tensorboard?logdir=" + encodeURIComponent('volume://data/notexit/mountnotexist/log-dir-1') + "&namespace=test-ns&tfversion=2.0.0")
                .expect(500, "Failed to start Tensorboard app: Cannot find file \"volume://data/notexit/mountnotexist/log-dir-1\" in pod \"unknown\": volume \"data\" not mounted or volume \"data\" with subPath (which is prefix of notexit/mountnotexist/log-dir-1) not mounted", function (err) {
                expect(errorSpy).toHaveBeenCalledTimes(1);
                done(err);
            });
        });
        it('returns error when there is an existing tensorboard with different version', function (done) {
            var errorSpy = jest.spyOn(console, 'error');
            errorSpy.mockImplementation();
            k8sGetCustomObjectSpy.mockImplementation(function () {
                return Promise.resolve(newGetTensorboardResponse({
                    name: 'viewer-abcdefg',
                    logDir: 'log-dir-1',
                    tensorflowImage: 'tensorflow:2.1.0',
                }));
            });
            k8sCreateCustomObjectSpy.mockImplementation(function () { return Promise.resolve(); });
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .post("/apps/tensorboard?logdir=" + encodeURIComponent('log-dir-1') + "&namespace=test-ns&tfversion=2.0.0")
                .expect(500, "Failed to start Tensorboard app: There's already an existing tensorboard instance with a different version 2.1.0", function (err) {
                expect(errorSpy).toHaveBeenCalledTimes(1);
                done(err);
            });
        });
        it('returns existing pod address if there is an existing tensorboard with the same version', function (done) {
            k8sGetCustomObjectSpy.mockImplementation(function () {
                return Promise.resolve(newGetTensorboardResponse({
                    name: 'viewer-abcdefg',
                    logDir: 'log-dir-1',
                    tensorflowImage: 'tensorflow:2.0.0',
                }));
            });
            k8sCreateCustomObjectSpy.mockImplementation(function () { return Promise.resolve(); });
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .post("/apps/tensorboard?logdir=" + encodeURIComponent('log-dir-1') + "&namespace=test-ns&tfversion=2.0.0")
                .expect(200, 'http://viewer-abcdefg-service.test-ns.svc.cluster.local:80/tensorboard/viewer-abcdefg/', done);
        });
    });
    describe('delete', function () {
        it('requires logdir', function (done) {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .delete('/apps/tensorboard')
                .expect(400, 'logdir argument is required', done);
        });
        it('requires namespace', function (done) {
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .delete('/apps/tensorboard?logdir=some-log-dir')
                .expect(400, 'namespace argument is required', done);
        });
        it('deletes tensorboard viewer custom object', function (done) {
            k8sGetCustomObjectSpy.mockImplementation(function () {
                return Promise.resolve(newGetTensorboardResponse({
                    name: 'viewer-abcdefg',
                    logDir: 'log-dir-1',
                    tensorflowImage: 'tensorflow:2.0.0',
                }));
            });
            k8sDeleteCustomObjectSpy.mockImplementation(function () { return Promise.resolve(); });
            app = new app_1.UIServer(configs_1.loadConfigs(argv, {}));
            supertest_1.default(app.start())
                .delete("/apps/tensorboard?logdir=" + encodeURIComponent('log-dir-1') + "&namespace=test-ns")
                .expect(200, 'Tensorboard deleted.', function (err) {
                expect(k8sDeleteCustomObjectSpy.mock.calls[0]).toMatchInlineSnapshot("\n              Array [\n                \"kubeflow.org\",\n                \"v1beta1\",\n                \"test-ns\",\n                \"viewers\",\n                \"viewer-5e1404e679e27b0f0b8ecee8fe515830eaa736c5\",\n                V1DeleteOptions {},\n              ]\n            ");
                done(err);
            });
        });
    });
});
//# sourceMappingURL=tensorboard.test.js.map