"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright 2020 The Kubeflow Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var k8s_helper_1 = require("./k8s-helper");
describe('k8s-helper', function () {
    describe('parseTensorboardLogDir', function () {
        var podTemplateSpec = {
            spec: {
                containers: [
                    {
                        volumeMounts: [
                            {
                                name: 'output',
                                mountPath: '/data',
                            },
                            {
                                name: 'artifact',
                                subPath: 'pipeline1',
                                mountPath: '/data1',
                            },
                            {
                                name: 'artifact',
                                subPath: 'pipeline2',
                                mountPath: '/data2',
                            },
                        ],
                    },
                ],
                volumes: [
                    {
                        name: 'output',
                        hostPath: {
                            path: '/data/output',
                            type: 'Directory',
                        },
                    },
                    {
                        name: 'artifact',
                        persistentVolumeClaim: {
                            claimName: 'artifact_pvc',
                        },
                    },
                ],
            },
        };
        it('handles not volume storage', function () {
            var logdir = 'gs://testbucket/test/key/path';
            var url = k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec);
            expect(url).toEqual(logdir);
        });
        it('handles not volume storage with Series', function () {
            var logdir = 'Series1:gs://testbucket/test/key/path1,Series2:gs://testbucket/test/key/path2';
            var url = k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec);
            expect(url).toEqual(logdir);
        });
        it('handles volume storage without subPath', function () {
            var logdir = 'volume://output';
            var url = k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec);
            expect(url).toEqual('/data');
        });
        it('handles volume storage without subPath with Series', function () {
            var logdir = 'Series1:volume://output/volume/path1,Series2:volume://output/volume/path2';
            var url = k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec);
            expect(url).toEqual('Series1:/data/volume/path1,Series2:/data/volume/path2');
        });
        it('handles volume storage with subPath', function () {
            var logdir = 'volume://artifact/pipeline1';
            var url = k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec);
            expect(url).toEqual('/data1');
        });
        it('handles volume storage with subPath with Series', function () {
            var logdir = 'Series1:volume://artifact/pipeline1/path1,Series2:volume://artifact/pipeline2/path2';
            var url = k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec);
            expect(url).toEqual('Series1:/data1/path1,Series2:/data2/path2');
        });
        it('handles volume storage without subPath throw volume not configured error', function () {
            var logdir = 'volume://other/path';
            expect(function () { return k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec); }).toThrowError('Cannot find file "volume://other/path" in pod "unknown": volume "other" not configured');
        });
        it('handles volume storage without subPath throw volume not configured error with Series', function () {
            var logdir = 'Series1:volume://output/volume/path1,Series2:volume://other/volume/path2';
            expect(function () { return k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec); }).toThrowError('Cannot find file "volume://other/volume/path2" in pod "unknown": volume "other" not configured');
        });
        it('handles volume storage without subPath throw volume not mounted', function () {
            var noMountPodTemplateSpec = {
                spec: {
                    volumes: [
                        {
                            name: 'artifact',
                            persistentVolumeClaim: {
                                claimName: 'artifact_pvc',
                            },
                        },
                    ],
                },
            };
            var logdir = 'volume://artifact/path1';
            expect(function () {
                return k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, noMountPodTemplateSpec);
            }).toThrowError('Cannot find file "volume://artifact/path1" in pod "unknown": container "" not found');
        });
        it('handles volume storage without volumeMounts throw volume not mounted', function () {
            var noMountPodTemplateSpec = {
                spec: {
                    containers: [
                        {
                            volumeMounts: [
                                {
                                    name: 'other',
                                    mountPath: '/data',
                                },
                            ],
                        },
                    ],
                    volumes: [
                        {
                            name: 'artifact',
                            persistentVolumeClaim: {
                                claimName: 'artifact_pvc',
                            },
                        },
                    ],
                },
            };
            var logdir = 'volume://artifact/path';
            expect(function () { return k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec); }).toThrowError('Cannot find file "volume://artifact/path" in pod "unknown": volume "artifact" not mounted');
        });
        it('handles volume storage with subPath throw volume mount not found', function () {
            var logdir = 'volume://artifact/other';
            expect(function () { return k8s_helper_1.TEST_ONLY.parseTensorboardLogDir(logdir, podTemplateSpec); }).toThrowError('Cannot find file "volume://artifact/other" in pod "unknown": volume "artifact" not mounted or volume "artifact" with subPath (which is prefix of other) not mounted');
        });
    });
});
//# sourceMappingURL=k8s-helper.test.js.map